/****************************************************************
 * Slider
 *   
 * Slider000118
 * by Christiaan Hofman, January 2000
 *   
 * You may use or modify this code provided that this copyright notice
 * appears on all copies.
 *   
 *  Constructor:
 *    var slider = new Slider( [name] );
 *   
 *  Defining Methods
 *    slider.writeSlider( [x, y] );
 * 
 *  Control Methods:
 *    slider.getValue();
 *    slider.setValue( [value] );
 *
 *  Event Handlers:
 *    slider.onmouseover( e, slider );
 *    slider.onmouseout( e, slider );
 *    slider.onmousedown( e, slider );
 *    slider.onmouseup( e, slider );
 *    slider.onslide( e, slider );
 *    slider.onchange( e, slider );
 *    slider.onclick( e, slider );
 * 
 *  Default Initial Settings:   
 * 
 *    slider.leftValue = 0;
 *    slider.rightValue = 1;
 *    slider.defaultValue = 0;
 *    slider.position = "relative";
 *    slider.border = 1;
 *    slider.railWidth = 300;
 *    slider.railHeight = 28;
 *    slider.buttonWidth = 40;
 *    slider.buttonHeight = 26;
 *    slider.railImg = "sliderbg.gif";
 *    slider.buttonImg = "sliderbutton.gif";
 *    slider.buttonHiliteImg = "sliderhibutton.gif";
 *    slider.imgPath = "";
 *    slider.orientation = "h";
 *    slider.noTables = true;
 * 
 ****************************************************************
 */

function Slider(name) {
    this.leftValue = 0;
    this.rightValue = 1;
    this.defaultValue = 0;
    this.position = "relative";
    this.border = 1;
    this.railWidth = 300;
    this.railHeight = 30;
    this.buttonWidth = 40;
    this.buttonHeight = 28;
    this.railImg = "sliderbg.gif";
    this.buttonImg = "sliderbutton.gif";
    this.buttonHiliteImg = "sliderhibutton.gif";
    this.imgPath = "";
    this.orientation = "h";
    
    this.writeSlider = Slider.writeSlider;
    this.makeEventHandler = Slider.makeEventHandler;
    this.isPrototype = Slider.isPrototype;
    this.getValue = Slider.getValue;
    this.setValue = Slider.setValue;
     
    this.MouseOver = Slider.MouseOver;
    this.MouseOut = Slider.MouseOut;
    this.MouseDown = Slider.MouseDown;
    this.MouseUp = Slider.MouseUp;
    this.MouseSlide = Slider.MouseSlide;

    this.onmouseover = null;
    this.onmouseout = null;
    this.onmousedown = null;
    this.onmouseup = null;
    this.onslide = null;
    this.onchange = null;
    this.onclick = null;

    this.noTables = false;

    if (!window.sliders)  window.sliders = new Array();
    this.name = name || "slider"+window.sliders.length;
    window.sliders[window.sliders.length] = this;
    window.sliders[this.name] = this;
    if (!window.sliderDrag)  window.sliderDrag = new Object();
}

Slider.writeSlider = function (x,y) {
    x = x || 0;
    y = y || 0;
    var proto = this.prototype || this;
    proto.buttonWidth = proto.buttonWidth;
    proto.buttonHeight = proto.buttonHeight;
    proto.sliderHeight = Math.max(proto.railHeight, proto.buttonHeight);
    proto.sliderWidth = Math.max(proto.railWidth, proto.buttonWidth);
    this.offset = proto.border;
    this.orientation = proto.orientation;
    
    this.loImg = new Image(proto.buttonWidth,proto.buttonHeight);
    this.hiImg = new Image(proto.buttonWidth,proto.buttonHeight);
    this.loImg.src = proto.imgPath + proto.buttonImg;
    this.hiImg.src = proto.imgPath + proto.buttonHiliteImg;

    if (proto.orientation == "v") {
        var bleft = (proto.sliderWidth-proto.buttonWidth)/2;
        var btop = proto.border;
        var rleft = (proto.sliderWidth-proto.railWidth)/2;
        var rtop = 0;
    } else {
        var bleft =  proto.border;
        var btop = (proto.sliderHeight-proto.buttonHeight)/2;
        var rleft = 0;
        var rtop = (proto.sliderHeight-proto.railHeight)/2;
    }
    var style = '<STYLE TYPE="text/css"><!--\n' +
        '#'+this.name+'Slider {position:'+ this.position +'; visibility:visible; left:'+x+'; top:'+y+'; width:'+ proto.railWidth +'; height:'+ proto.sliderHeight +'; clip:rect(0 '+ proto.sliderWidth +' '+ proto.sliderHeight +' 0); z-index:0;}\n' +
        '#'+this.name+'Rail {visibility:inherit; position:absolute; left:'+ rleft +'; top:'+ rtop +'; width:'+ proto.railWidth +'; height:'+ proto.railHeight +'; z-index:0; }\n' +
        '#'+this.name+'Button {visibility:inherit; position:absolute; left:'+ bleft +'; top:'+ btop +'; width:'+ proto.buttonWidth +'; height:'+ proto.buttonHeight +'; z-index:1; }\n' +
        '--></STYLE>\n';

    var content = '<SPAN ID="'+this.name+'Slider">\n' +
        '  <DIV ID="'+this.name+'Rail"><IMG ID="'+this.name+'RailImg" SRC="'+proto.imgPath+proto.railImg+'"></DIV>\n' +
        '  <DIV ID="'+this.name+'Button"><IMG ID="'+this.name+'ButtonImg" SRC="'+proto.imgPath+proto.buttonImg+'"></DIV>\n' +
        '</SPAN>\n';

    if (proto.position == "relative") {
        if (this.noTable == "cell" || !this.noTable) {
            var tableHeight = proto.sliderHeight + y;
            var tableWidth = proto.sliderWidth + Math.max(x,4);
            content = '<TD VALIGN="top" WIDTH="'+ tableWidth +'" HEIGHT="'+ tableHeight +'">\n'+ content +'</TD>';
        }
        if (!this.noTable && this.noTable != "cell") {
            content = '<TABLE CELLPADDING=0 CELLSPACING=0><TR>'+ content +'</TR></TABLE>\n';
        }
    }

    document.writeln(style);
    document.writeln(content);
    if (document.layers) {
        this.sliderLayer = document.layers[this.name+"Slider"];
        this.rail = this.sliderLayer.layers[this.name+"Rail"];
        this.button = this.sliderLayer.layers[this.name+"Button"];
        this.button.img = this.button.document.images[0];
        
        this.sliderLayer.clip.width = proto.sliderWidth;
        this.sliderLayer.clip.height = proto.sliderHeight;
        this.rail.clip.width = proto.railWidth;
        this.rail.clip.height = proto.railHeight;
        this.button.clip.width = proto.buttonWidth;
        this.button.clip.height = proto.buttonHeight;
        if (proto.orientation == "h") {
            this.offset = this.rail.left + proto.border;
            this.maxSlide = this.rail.clip.width - this.button.clip.width - 2 * proto.border;
        } else {
            this.offset = this.rail.top + proto.border;
            this.maxSlide = this.rail.clip.height - this.button.clip.height - 2 * proto.border;
        }
        this.button.captureEvents(Event.MOUSEOVER|Event.MOUSEDOWN|Event.MOUSEOUT);
        this.button.onmousedown = this.MouseDown;
        this.button.onmouseout = this.MouseOut;
        this.button.onmouseover = this.MouseOver;
    } else if (document.all) { 
        this.sliderLayer = document.all[this.name+"Slider"].style;
        this.rail = document.all[this.name+"Rail"].style;
        this.button = document.all[this.name+"Button"];
        this.button.img = document.all[this.name+"ButtonImg"];

        this.sliderLayer.pixelWidth = proto.sliderWidth;
        this.sliderLayer.pixelHeight = proto.sliderHeight;
        this.rail.pixelWidth = proto.railWidth;
        this.rail.pixelHeight = proto.railHeight;
        this.button.style.pixelWidth = proto.buttonWidth;
        this.button.style.pixelHeight = proto.buttonHeight;
        if (proto.orientation == "h") {
            this.offset = this.rail.pixelLeft + proto.border;
            this.maxSlide = this.rail.pixelWidth - this.button.style.pixelWidth - 2 * proto.border;
        } else {
            this.offset = this.rail.pixelTop + proto.border;
            this.maxSlide = this.rail.pixelHeight - this.button.style.pixelHeight - 2 * proto.border;
        }
        this.button.onmousedown = this.MouseDown;
        this.button.onmouseout = this.MouseOut;
        this.button.onmouseover = this.MouseOver;
    } else if (document.getElementById) { 
        this.sliderLayer = document.getElementById(this.name+"Slider").style;
        this.rail = document.getElementById(this.name+"Rail").style;
        this.button = document.getElementById(this.name+"Button");
        this.button.img = document.getElementById(this.name+"ButtonImg");

        this.sliderLayer.width = proto.sliderWidth; +"px"
        this.sliderLayer.height = proto.sliderHeight +"px";
        this.rail.width = proto.railWidth +"px";
        this.rail.height = proto.railHeight +"px";
        this.button.style.width = proto.buttonWidth +"px";
        this.button.style.height = proto.buttonHeight +"px";
        if (proto.orientation == "h") {
            this.offset = parseInt(this.rail.left) + proto.border;
            this.maxSlide = parseInt(this.rail.width) - parseInt(this.button.style.width) - 2 * proto.border;
        } else {
            this.offset = parseInt(this.rail.top) + proto.border;
            this.maxSlide = parseInt(this.rail.height) - parseInt(this.button.style.height) - 2 * proto.border;
        }
        this.button.addEventListener("mousedown",this.MouseDown,false);
        this.button.addEventListener("mouseout", this.MouseOut,false);
        this.button.addEventListener("mouseover",this.MouseOver,false);
    }
    this.button.slider = this;
    this.onmouseover = this.makeEventHandler(this.onmouseover);
    this.onmouseout  = this.makeEventHandler(this.onmouseout);
    this.onmousedown = this.makeEventHandler(this.onmousedown);
    this.onmouseup   = this.makeEventHandler(this.onmouseup);
    this.onslide     = this.makeEventHandler(this.onslide);
    this.onchange    = this.makeEventHandler(this.onchange);
    this.onclick     = this.makeEventHandler(this.onclick);
    if (document.all || document.getElementById) this.button = this.button.style;
    this.setValue(this.defaultValue,true);
    this.sliderLayer.visibility = "visible";
}

Slider.isPrototype = function () {
    for (var i=0; i<window.sliders.length; i++)  
        window.sliders[i].prototype = window.sliders[i].prototype || this;
}

Slider.MouseOver = function (e) {
    if (this.slider.onmouseover)  this.slider.onmouseover(e);
}

Slider.MouseOut = function (e) {
    if (this.slider.onmouseout)  this.slider.onmouseout(e);
}

Slider.MouseDown = function (e) {
    var slider = this.slider;
    window.sliderDrag.dragLayer = this;
    window.sliderDrag.dragged = false;
    if (document.layers) {
        if (e.which > 1)  return true;
        window.sliderDrag.offX = e.pageX - this.left + slider.offset;
        window.sliderDrag.offY = e.pageY - this.top + slider.offset;
    } else if (document.all) {
        if (window.event.button > 1)  return true;
        window.sliderDrag.offX  =  window.event.clientX - this.style.pixelLeft + slider.offset;
        window.sliderDrag.offY  =  window.event.clientY - this.style.pixelTop + slider.offset;
        window.event.cancelBubble = true;
    } else if (document.getElementById) {
        if (e.button > 0)  return true;
        window.sliderDrag.offX  =  e.pageX - parseInt(this.style.left) + slider.offset;
        window.sliderDrag.offY  =  e.pageY - parseInt(this.style.top) + slider.offset;
        e.stopPropagation();
    }
    if (document.captureEvents)  document.captureEvents(Event.MOUSEMOVE|Event.MOUSEUP);
    window.sliderDrag.savemousemove = document.onmousemove;
    document.onmousemove = slider.MouseSlide;
    window.sliderDrag.savemouseup = document.onmouseup;
    document.onmouseup = slider.MouseUp;
    this.img.src = slider.hiImg.src;
    if (slider.onmousedown)  slider.onmousedown(e);
    return false;
}

Slider.MouseUp = function (e) {
    var l = window.sliderDrag.dragLayer;
    var slider = l.slider;
    document.onmousemove = null;
    document.onmouseup = null;
    if (document.releaseEvents) document.releaseEvents();
    if (e && e.stopPropagation) e.stopPropagation();
    window.sliderDrag.dragLayer = null;
    l.img.src = slider.loImg.src;
    if (slider.onchange)  slider.onchange(e);
    if (slider.onmouseup)  slider.onmouseup(e);
    if (!window.sliderDrag.dragged && slider.onclick) slider.onclick(e);
    return false;
}

Slider.MouseSlide = function (e) {
    var l = window.sliderDrag.dragLayer;
    var slider = l.slider;
    window.sliderDrag.dragged = true;
    if (document.layers) {
        if (slider.orientation == "h")
            l.left = Math.max(Math.min(e.pageX - window.sliderDrag.offX,slider.maxSlide),0) + slider.offset;
        else  l.top = Math.max(Math.min(e.pageY - window.sliderDrag.offY,slider.maxSlide),0) + slider.offset;
    } else if (document.all) {
        if (slider.orientation == "h")
            l.style.pixelLeft = Math.max(Math.min(window.event.clientX - window.sliderDrag.offX,slider.maxSlide),0) + slider.offset;
        else  l.style.pixelTop = Math.max(Math.min(window.event.clientY - window.sliderDrag.offY,slider.maxSlide),0) + slider.offset;
    } else if (document.getElementById) {
        if (slider.orientation == "h")
            l.style.left = Math.max(Math.min(e.pageX + window.sliderDrag.offX,slider.maxSlide),0) + slider.offset +"px";
        else  l.style.top = Math.max(Math.min(e.pageY + window.sliderDrag.offY,slider.maxSlide),0) + slider.offset +"px";
    }
    if (slider.onchange)  slider.onchange(e);
    if (slider.onslide)  slider.onslide(e);
    return false;
}

Slider.getValue = function () {
    var pos;
    if (document.layers) {
        pos = (this.orientation == "h")? this.button.left : this.button.top;
    } else if (document.all) {
        pos = (this.orientation == "h")? this.button.pixelLeft : this.button.pixelTop;
    } else if (document.getElementById) {
        pos = (this.orientation == "h")? parseInt(this.button.left) : parseInt(this.button.top);
    }
    return  this.leftValue + (this.rightValue-this.leftValue) * (pos-this.offset) / this.maxSlide;
}

Slider.setValue = function (value,ignore) {
    if (typeof(value) == "string")  value = parseFloat(value);
    if (isNaN(value))  value = this.defaultValue;
    var rangeValue = (this.rightValue >= this.leftValue)? 
        Math.min(Math.max(value,this.leftValue),this.rightValue) - this.leftValue : 
        Math.max(Math.min(value,this.leftValue),this.rightValue) - this.leftValue;
    var pos = parseInt(this.maxSlide * rangeValue / (this.rightValue-this.leftValue)) + this.offset;
    if (document.layers) {
        if (this.orientation == "h") this.button.left = pos;
        else  this.button.top = pos;
    } else if (document.all) {
        if (this.orientation == "h") this.button.pixelLeft = pos;
        else  this.button.pixelTop = pos;
    } else if (document.getElementById) {
        if (this.orientation == "h") this.button.left = pos +"px";
        else  this.button.top = pos +"px";
    }
    if (this.onchange && (!ignore))  this.onchange(null);
}

Slider.makeEventHandler = function (f) {
    return (typeof(f) == "string")? new Function('e',f) : f;
}

function toDecimals(val,n) {
    if (isNaN(n)) return val;
	   if (n<=0) return Math.round(val);
    for (var m=0; m<n; m++)  val *= 10;
    val = Math.round(val);
    valstr = val.toString();
    len = valstr.length;
    if (len>n)
        valstr = valstr.substring(0,len-n) +"."+ valstr.substring(len-n,len);
    else {
        while (valstr.length<n) valstr = "0"+valstr;
        valstr = "0."+valstr;
    }
    return valstr;
}

